Package: spatstat.linnet
Version: 3.0-6
Date: 2023-02-20
Title: Linear Networks Functionality of the 'spatstat' Family
Authors@R: c(person("Adrian", "Baddeley", 
                    role = c("aut", "cre", "cph"),
       	            email = "Adrian.Baddeley@curtin.edu.au",
		    comment = c(ORCID="0000-0001-9499-8382")),
	     person("Rolf", "Turner", 
                    role = c("aut", "cph"),
 	            email="r.turner@auckland.ac.nz",
		    comment=c(ORCID="0000-0001-5521-5218")),
	     person("Ege",   "Rubak", 
                    role = c("aut", "cph"),
		    email = "rubak@math.aau.dk",
		    comment=c(ORCID="0000-0002-6675-533X")),
	     person("Greg", "McSwiggan",
	            role = c("aut", "cph")),
	     person("Tilman", "Davies",
	            role = c("ctb", "cph")),
	     person("Mehdi", "Moradi",
	            role = c("ctb", "cph")),
	     person("Suman", "Rakshit",
	            role = c("ctb", "cph")),
	     person("Ottmar", "Cronie",
	            role = "ctb"))
Maintainer: Adrian Baddeley <Adrian.Baddeley@curtin.edu.au>
Depends: R (>= 3.5.0), spatstat.data (>= 3.0), spatstat.geom (>=
        3.0-6), spatstat.random (>= 3.1-3), spatstat.explore (>=
        3.0-6), spatstat.model (>= 3.2-1), stats, graphics, grDevices,
        methods, utils
Imports: spatstat.utils (>= 3.0), Matrix, spatstat.sparse (>= 3.0)
Suggests: goftest, locfit, spatstat (>= 3.0)
Additional_repositories: https://spatstat.r-universe.dev
Description: Defines types of spatial data on a linear network
	     and provides functionality for geometrical operations,
	     data analysis and modelling of data on a linear network,
	     in the 'spatstat' family of packages.
	     Contains definitions and support for linear networks, including creation of networks, geometrical measurements, topological connectivity, geometrical operations such as inserting and deleting vertices, intersecting a network with another object, and interactive editing of networks.
	     Data types defined on a network include point patterns, pixel images, functions, and tessellations.
	     Exploratory methods include kernel estimation of intensity on a network, K-functions and pair correlation functions on a network, simulation envelopes, nearest neighbour distance and empty space distance, relative risk estimation with cross-validated bandwidth selection. Formal hypothesis tests of random pattern (chi-squared, Kolmogorov-Smirnov, Monte Carlo, Diggle-Cressie-Loosmore-Ford, Dao-Genton, two-stage Monte Carlo) and tests for covariate effects (Cox-Berman-Waller-Lawson, Kolmogorov-Smirnov, ANOVA) are also supported.
	Parametric models can be fitted to point pattern data using the function lppm() similar to glm(). Only Poisson models are implemented so far. Models may involve dependence on covariates and dependence on marks. Models are fitted by maximum likelihood.
	Fitted point process models can be simulated, automatically. Formal hypothesis tests of a fitted model are supported (likelihood ratio test, analysis of deviance, Monte Carlo tests) along with basic tools for model selection (stepwise(), AIC()) and variable selection (sdr). Tools for validating the fitted model include simulation envelopes, residuals, residual plots and Q-Q plots, leverage and influence diagnostics, partial residuals, and added variable plots.
	Random point patterns on a network can be generated using a variety of models.
License: GPL (>= 2)
URL: http://spatstat.org/
NeedsCompilation: yes
ByteCompile: true
BugReports: https://github.com/spatstat/spatstat.linnet/issues
Packaged: 2023-02-20 03:51:48 UTC; adrian
Author: Adrian Baddeley [aut, cre, cph]
    (<https://orcid.org/0000-0001-9499-8382>),
  Rolf Turner [aut, cph] (<https://orcid.org/0000-0001-5521-5218>),
  Ege Rubak [aut, cph] (<https://orcid.org/0000-0002-6675-533X>),
  Greg McSwiggan [aut, cph],
  Tilman Davies [ctb, cph],
  Mehdi Moradi [ctb, cph],
  Suman Rakshit [ctb, cph],
  Ottmar Cronie [ctb]
Repository: CRAN
Date/Publication: 2023-02-22 01:10:02 UTC
